@@ -1,23 +1,3 @@
-/**
-  Copyright Red Hat, Inc. 2006
-
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2, or (at your option) any
-  later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; see the file COPYING.  If not, write to the
-  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
-  MA 02139, USA.
-
-  Author: Lon Hohberger <lhh at redhat.com>
- */
 /**
   @file Main loop / functions for disk-based quorum daemon.
  */
@@ -36,6 +16,7 @@
 #include <time.h>
 #include <sys/reboot.h>
 #include <sys/time.h>
+#include <sys/un.h>
 #include <linux/reboot.h>
 #include <sched.h>
 #include <signal.h>
@@ -47,9 +28,13 @@
 #include <cluster/cnxman-socket.h>
 #endif
 
+/* from daemon_init.c */
 int daemon_init(char *);
 int check_process_running(char *, pid_t *);
 
+/* from proc.c */
+char *state_str(disk_node_state_t s);
+
 /*
   TODO:
   1) Take into account timings to gracefully extend node timeouts during 
@@ -67,7 +52,7 @@ inline void _diff_tv(struct timeval *dest, struct timeval *start,
 		     struct timeval *end);
 
 static int _running = 1;
-void update_local_status(qd_ctx *ctx, node_info_t *ni, int max, int score,
+static void update_local_status(qd_ctx *ctx, node_info_t *ni, int max, int score,
 		    	 int score_req, int score_max);
 
 
@@ -81,7 +66,7 @@ int_handler(int sig)
 /**
   Simple thing to see if a node is running.
  */
-inline int
+static inline int
 state_run(disk_node_state_t state)
 {
 	return (state >= S_INIT ? state : 0);
@@ -91,7 +76,7 @@ state_run(disk_node_state_t state)
 /**
   Clear out / initialize node info block.
  */
-void
+static void
 node_info_init(node_info_t *ni, int max)
 {
 	int x;
@@ -112,7 +97,7 @@ node_info_init(node_info_t *ni, int max)
   Rare case; usually other nodes would put up the 'Undead' message and
   re-evict us.
  */
-void
+static void
 check_self(qd_ctx *ctx, status_block_t *sb)
 {
 	if (!sb->ps_updatenode ||
@@ -137,20 +122,22 @@ check_self(qd_ctx *ctx, status_block_t *sb)
   or has not updated their timestamp recently.  See check_transitions as
   well.
  */
-void
+int
 read_node_blocks(qd_ctx *ctx, node_info_t *ni, int max)
 {
-	int x;
+	int x, errors = 0;
 	status_block_t *sb;
 
 	for (x = 0; x < max; x++) {
 
 		sb = &ni[x].ni_status;
 
-		if (qdisk_read(ctx->qc_fd, qdisk_nodeid_offset(x+1),
+		if (qdisk_read(&ctx->qc_disk,
+			       qdisk_nodeid_offset(x+1, ctx->qc_disk.d_blksz),
 			       sb, sizeof(*sb)) < 0) {
 			clulog(LOG_WARNING,"Error reading node ID block %d\n",
 			       x+1);
+			++errors;
 			continue;
 		}
 		swab_status_block_t(sb);
@@ -186,13 +173,15 @@ read_node_blocks(qd_ctx *ctx, node_info_t *ni, int max)
 		ni[x].ni_seen++;
 		ni[x].ni_last_seen = sb->ps_timestamp;
 	}
+
+	return errors;
 }
 
 
 /**
   Check for node transitions.
  */
-void
+static void
 check_transitions(qd_ctx *ctx, node_info_t *ni, int max, memb_mask_t mask)
 {
 	int x;
@@ -251,23 +240,6 @@ check_transitions(qd_ctx *ctx, node_info_t *ni, int max, memb_mask_t mask)
 		if (ni[x].ni_misses > ctx->qc_tko &&
 		     state_run(ni[x].ni_status.ps_state)) {
 
-			/*
-			   Write eviction notice if we're the master.
-			 */
-			if (ctx->qc_status == S_MASTER) {
-				clulog(LOG_NOTICE,
-				       "Writing eviction notice for node %d\n",
-				       ni[x].ni_status.ps_nodeid);
-				qd_write_status(ctx, ni[x].ni_status.ps_nodeid,
-						S_EVICT, NULL, NULL, NULL);
-				if (ctx->qc_flags & RF_ALLOW_KILL) {
-					clulog(LOG_DEBUG, "Telling CMAN to "
-						"kill the node\n");
-					cman_kill_node(ctx->qc_ch,
-						ni[x].ni_status.ps_nodeid);
-				}
-			}
-
 			/*
 			   Mark our internal views as dead if nodes miss too
 			   many heartbeats...  This will cause a master
@@ -285,6 +257,23 @@ check_transitions(qd_ctx *ctx, node_info_t *ni, int max, memb_mask_t mask)
 			ni[x].ni_evil_incarnation = 
 				ni[x].ni_status.ps_incarnation;
 			
+			/*
+			   Write eviction notice if we're the master.
+			 */
+			if (ctx->qc_status == S_MASTER) {
+				clulog(LOG_NOTICE,
+				       "Writing eviction notice for node %d\n",
+				       ni[x].ni_status.ps_nodeid);
+				qd_write_status(ctx, ni[x].ni_status.ps_nodeid,
+						S_EVICT, NULL, NULL, NULL);
+				if (ctx->qc_flags & RF_ALLOW_KILL) {
+					clulog(LOG_DEBUG, "Telling CMAN to "
+						"kill the node\n");
+					cman_kill_node(ctx->qc_ch,
+						ni[x].ni_status.ps_nodeid);
+				}
+			}
+
 			/* Clear our master mask for the node after eviction */
 			if (mask)
 				clear_bit(mask, (ni[x].ni_status.ps_nodeid-1),
@@ -380,7 +369,7 @@ check_transitions(qd_ctx *ctx, node_info_t *ni, int max, memb_mask_t mask)
   Checks for presence of an online master.  If there is no
   Returns
  */
-int
+static int
 master_exists(qd_ctx *ctx, node_info_t *ni, int max, int *low_id, int *count)
 {
 	int x;
@@ -448,10 +437,10 @@ master_exists(qd_ctx *ctx, node_info_t *ni, int max, int *low_id, int *count)
   a cluster running using this QD.  Note that this will delay master
   election if multiple nodes start with a second or two of each other.
  */
-int
+static int
 quorum_init(qd_ctx *ctx, node_info_t *ni, int max, struct h_data *h, int maxh)
 {
-	int x = 0, score, maxscore, score_req;
+	int x = 0, score, maxscore, score_req = 0;
 
 	clulog(LOG_INFO, "Quorum Daemon Initializing\n");
 	
@@ -462,12 +451,14 @@ quorum_init(qd_ctx *ctx, node_info_t *ni, int max, struct h_data *h, int maxh)
 	if (qdisk_validate(ctx->qc_device) < 0)
 		return -1;
 
-	ctx->qc_fd = qdisk_open(ctx->qc_device);
-	if (ctx->qc_fd < 0) {
+	if (qdisk_open(ctx->qc_device, &ctx->qc_disk) < 0) {
 		clulog(LOG_CRIT, "Failed to open %s: %s\n", ctx->qc_device,
 		       strerror(errno));
 		return -1;
 	}
+
+	clulog(LOG_DEBUG, "I/O Size: %d  Page Size: %d\n",
+	       ctx->qc_disk.d_blksz, ctx->qc_disk.d_pagesz);
 	
 	if (h && maxh) {
 		start_score_thread(ctx, h, maxh);
@@ -505,6 +496,8 @@ quorum_init(qd_ctx *ctx, node_info_t *ni, int max, struct h_data *h, int maxh)
 
 	get_my_score(&score, &maxscore);
 	clulog(LOG_INFO, "Initial score %d/%d\n", score, maxscore);
+	if ((ctx->qc_flags & RF_STOP_CMAN) && (score < score_req))
+		return -1;
 	clulog(LOG_INFO, "Initialization complete\n");
 
 	return 0;
@@ -514,7 +507,7 @@ quorum_init(qd_ctx *ctx, node_info_t *ni, int max, struct h_data *h, int maxh)
 /**
   Vote for a master if it puts a bid in.
  */
-void
+static void
 do_vote(qd_ctx *ctx, node_info_t *ni, int max, disk_msg_t *msg)
 {
 	int x;
@@ -542,7 +535,7 @@ do_vote(qd_ctx *ctx, node_info_t *ni, int max, disk_msg_t *msg)
   Check to match nodes in mask with nodes online according to CMAN.
   Only the master needs to do this.
  */
-void
+static void
 check_cman(qd_ctx *ctx, memb_mask_t mask, memb_mask_t master_mask)
 {
 	cman_node_t nodes[MAX_NODES_DISK];
@@ -578,7 +571,7 @@ check_cman(qd_ctx *ctx, memb_mask_t mask, memb_mask_t master_mask)
 	*msg - it will store the vote for the lowest bid if we should
 	clear our bid.
  */ 
-int
+static int
 check_votes(qd_ctx *ctx, node_info_t *ni, int max, disk_msg_t *msg)
 {
 	int x, running = 0, acks = 0, nacks = 0, low_id = ctx->qc_my_id;
@@ -620,28 +613,7 @@ check_votes(qd_ctx *ctx, node_info_t *ni, int max, disk_msg_t *msg)
 	return 0;
 }
 
-
-char *
-state_str(disk_node_state_t s)
-{
-	switch (s) {
-	case S_NONE:
-		return "None";
-	case S_EVICT:
-		return "Evicted";
-	case S_INIT:
-		return "Initializing";
-	case S_RUN:
-		return "Running";
-	case S_MASTER:
-		return "Master";
-	default:
-		return "ILLEGAL";
-	}
-}
-
-
-void
+static void
 print_node_info(FILE *fp, node_info_t *ni)
 {
 	fprintf(fp, "node_info_t [node %d] {\n", ni->ni_status.ps_nodeid);
@@ -670,7 +642,7 @@ print_node_info(FILE *fp, node_info_t *ni)
 }
 
 
-void
+static void
 update_local_status(qd_ctx *ctx, node_info_t *ni, int max, int score,
 		    int score_req, int score_max)
 {
@@ -750,36 +722,6 @@ out:
 		fclose(fp);
 }
 
-
-/* Timeval functions from clumanager */
-/**
- * Scale a (struct timeval).
- *
- * @param tv		The timeval to scale.
- * @param scale		Positive multiplier.
- * @return		tv
- */
-struct timeval *
-_scale_tv(struct timeval *tv, int scale)
-{
-	tv->tv_sec *= scale;
-	tv->tv_usec *= scale;
-
-	if (tv->tv_usec > 1000000) {
-		tv->tv_sec += (tv->tv_usec / 1000000);
-		tv->tv_usec = (tv->tv_usec % 1000000);
-	}
-
-	return tv;
-}
-
-
-
-#define _print_tv(val) \
-	printf("%s: %d.%06d\n", #val, (int)((val)->tv_sec), \
-			(int)((val)->tv_usec))
-
-
 static inline int
 _cmp_tv(struct timeval *left, struct timeval *right)
 {
@@ -825,7 +767,7 @@ set_priority(int queue, int prio)
 }
 
 
-int
+static int
 cman_alive(cman_handle_t ch)
 {
 	fd_set rfds;
@@ -845,12 +787,12 @@ cman_alive(cman_handle_t ch)
 }
 
 
-int
+static int
 quorum_loop(qd_ctx *ctx, node_info_t *ni, int max)
 {
 	disk_msg_t msg = {0, 0, 0};
 	int low_id, bid_pending = 0, score, score_max, score_req,
-	    upgrade = 0, count;
+	    upgrade = 0, count, errors, error_cycles = 0;
 	memb_mask_t mask, master_mask;
 	struct timeval maxtime, oldtime, newtime, diff, sleeptime, interval;
 
@@ -875,7 +817,7 @@ quorum_loop(qd_ctx *ctx, node_info_t *ni, int max)
 		get_time(&oldtime, (ctx->qc_flags&RF_UPTIME));
 		
 		/* Read everyone else's status */
-		read_node_blocks(ctx, ni, max);
+		errors = read_node_blocks(ctx, ni, max);
 
 		/* Check for node transitions */
 		check_transitions(ctx, ni, max, mask);
@@ -912,7 +854,7 @@ quorum_loop(qd_ctx *ctx, node_info_t *ni, int max)
 				if (ctx->qc_flags & RF_REBOOT)
 					reboot(RB_AUTOBOOT);
 			}
-		}  else {
+		} else {
 			set_bit(mask, (ctx->qc_my_id-1), sizeof(mask));
 			if (ctx->qc_status == S_NONE) {
 				clulog(LOG_NOTICE,
@@ -1019,7 +961,8 @@ quorum_loop(qd_ctx *ctx, node_info_t *ni, int max)
 				return -1;
 			}
 			check_cman(ctx, mask, master_mask);
-			cman_poll_quorum_device(ctx->qc_ch, 1);
+			if (!errors)
+				cman_poll_quorum_device(ctx->qc_ch, 1);
 
 		} else if (ctx->qc_status == S_RUN && ctx->qc_master &&
 			   ctx->qc_master != ctx->qc_my_id) {
@@ -1038,7 +981,8 @@ quorum_loop(qd_ctx *ctx, node_info_t *ni, int max)
 						"Halting qdisk operations\n");
 					return -1;
 				}
-				cman_poll_quorum_device(ctx->qc_ch, 1);
+				if (!errors)
+					cman_poll_quorum_device(ctx->qc_ch, 1);
 			}
 		}
 		
@@ -1046,6 +990,8 @@ quorum_loop(qd_ctx *ctx, node_info_t *ni, int max)
 		if (qd_write_status(ctx, ctx->qc_my_id, ctx->qc_status,
 				    &msg, mask, master_mask) != 0) {
 			clulog(LOG_ERR, "Error writing to quorum disk\n");
+			errors++; /* this value isn't really used 
+				     at this point */
 		}
 
 		/* write out our local status */
@@ -1086,20 +1032,31 @@ quorum_loop(qd_ctx *ctx, node_info_t *ni, int max)
 			       (int)diff.tv_sec, (int)diff.tv_usec);
 			memcpy(&sleeptime, &interval, sizeof(sleeptime));
 		}
+
+		if (errors && ctx->qc_max_error_cycles) {
+			++error_cycles;
+			if (error_cycles >= ctx->qc_max_error_cycles) {
+				clulog(LOG_ALERT,
+				       "Too many I/O errors; giving up.\n");
+				_running = 0;
+			}
+		} else {
+			error_cycles = 0;
+		}
 		
 		/* Could hit a watchdog timer here if we wanted to */
 		if (_running)
 			select(0, NULL, NULL, NULL, &sleeptime);
 	}
 
-	return 0;
+	return !!errors;
 }
 
 
 /**
   Tell the other nodes we're done (safely!).
  */
-int
+static int
 quorum_logout(qd_ctx *ctx)
 {
 	/* Write out our status */
@@ -1115,8 +1072,8 @@ quorum_logout(qd_ctx *ctx)
 /**
   Grab all our configuration data from CCSD
  */
-int
-get_config_data(char *cluster_name, qd_ctx *ctx, struct h_data *h, int maxh,
+static int
+get_config_data(qd_ctx *ctx, struct h_data *h, int maxh,
 		int *cfh, int debug)
 {
 	int ccsfd = -1, loglevel = 4;
@@ -1125,7 +1082,7 @@ get_config_data(char *cluster_name, qd_ctx *ctx, struct h_data *h, int maxh,
 
 	clulog(LOG_DEBUG, "Loading configuration information\n");
 
-	ccsfd = ccs_force_connect(cluster_name, 1);
+	ccsfd = ccs_connect();
 	if (ccsfd < 0) {
 		clulog(LOG_CRIT, "Connection to CCSD failed; cannot start\n");
 		return -1;
@@ -1140,6 +1097,7 @@ get_config_data(char *cluster_name, qd_ctx *ctx, struct h_data *h, int maxh,
 		ctx->qc_flags |= RF_DEBUG;
 	ctx->qc_sched = SCHED_RR;
 	ctx->qc_sched_prio = 1;
+	ctx->qc_max_error_cycles = 0;
 
 	/* Get log log_facility */
 	snprintf(query, sizeof(query), "/cluster/quorumd/@log_facility");
@@ -1210,30 +1168,14 @@ get_config_data(char *cluster_name, qd_ctx *ctx, struct h_data *h, int maxh,
 	}
 	if (ctx->qc_master_wait <= ctx->qc_tko_up)
 		ctx->qc_master_wait = ctx->qc_tko_up + 1;
-
+		
 	/* Get votes */
-
-	/* check if votes is set in cluster.conf */
 	snprintf(query, sizeof(query), "/cluster/quorumd/@votes");
 	if (ccs_get(ccsfd, query, &val) == 0) {
 		ctx->qc_votes = atoi(val);
 		free(val);
 		if (ctx->qc_votes < 0)
 			ctx->qc_votes = 0;
-	} else { /* if votes is not set, default to node_num - 1 */
-		int nodes = 0, error;
-		for (;;) {
-			error = ccs_get_list(ccsfd, "/cluster/clusternodes/child::*", &val);
-			if (error || !val)
-				break;
-
-			nodes++;
-		}
-		nodes--;
-		if (nodes < 0)
-			nodes = 0;
-
-		ctx->qc_votes = nodes;
 	}
 
 	/* Get device */
@@ -1254,6 +1196,12 @@ get_config_data(char *cluster_name, qd_ctx *ctx, struct h_data *h, int maxh,
 		ctx->qc_status_file = val;
 	}
 
+	/* Get status socket */
+	snprintf(query, sizeof(query), "/cluster/quorumd/@status_sock");
+	if (ccs_get(ccsfd, query, &val) == 0) {
+		ctx->qc_status_sockname = val;
+	}
+
 	/* Get min score */
 	snprintf(query, sizeof(query), "/cluster/quorumd/@min_score");
 	if (ccs_get(ccsfd, query, &val) == 0) {
@@ -1302,6 +1250,15 @@ get_config_data(char *cluster_name, qd_ctx *ctx, struct h_data *h, int maxh,
 			ctx->qc_flags &= ~RF_REBOOT;
 		free(val);
 	}
+
+	/* Get cman_label */
+	snprintf(query, sizeof(query), "/cluster/quorumd/@cman_label");
+	if (ccs_get(ccsfd, query, &val) == 0) {
+		if (strlen(val) > 0) {
+			ctx->qc_flags |= RF_CMAN_LABEL;
+			ctx->qc_cman_label = val;
+		}
+	}
 	
 	/*
 	 * Get flag to see if we're supposed to kill cman if qdisk is not 
@@ -1361,11 +1318,26 @@ get_config_data(char *cluster_name, qd_ctx *ctx, struct h_data *h, int maxh,
 		free(val);
 	}
 
+
+	/*
+	 * How many consecutive error cycles do we allow before
+	 * giving up?
+	 */
+	/* default = no max */
+	snprintf(query, sizeof(query), "/cluster/quorumd/@max_error_cycles");
+	if (ccs_get(ccsfd, query, &val) == 0) {
+		ctx->qc_max_error_cycles = atoi(val);
+		if (ctx->qc_max_error_cycles <= 0)
+			ctx->qc_max_error_cycles = 0;
+		free(val);
+	}
+
 	*cfh = configure_heuristics(ccsfd, h, maxh);
 
 	clulog(LOG_DEBUG,
 	       "Quorum Daemon: %d heuristics, %d interval, %d tko, %d votes\n",
 	       *cfh, ctx->qc_interval, ctx->qc_tko, ctx->qc_votes);
+	clulog(LOG_DEBUG, "Run Flags: %08x\n", ctx->qc_flags);
 
 	ccs_disconnect(ccsfd);
 
@@ -1373,7 +1345,7 @@ get_config_data(char *cluster_name, qd_ctx *ctx, struct h_data *h, int maxh,
 }
 
 
-void
+static void
 check_stop_cman(qd_ctx *ctx)
 {
 	if (!(ctx->qc_flags & RF_STOP_CMAN))
@@ -1400,21 +1372,22 @@ int
 main(int argc, char **argv)
 {
 	cman_node_t me;
-	int cfh, rv, forked = 0, nfd = -1;
+	int cfh, rv, forked = 0, nfd = -1, ret = -1;
 	qd_ctx ctx;
-	cman_handle_t ch;
+	cman_handle_t ch = NULL;
 	node_info_t ni[MAX_NODES_DISK];
 	struct h_data h[10];
-	char debug = 0, foreground = 0;
+	int debug = 0, foreground = 0;
 	char device[128];
 	pid_t pid;
+	quorum_header_t qh;
 
 	if (check_process_running(argv[0], &pid) && pid !=getpid()) {
 		printf("QDisk services already running\n");
 		return 0;
 	}
 	
-	while ((rv = getopt(argc, argv, "fdQ")) != EOF) {
+	while ((rv = getopt(argc, argv, "fdQs")) != EOF) {
 		switch (rv) {
 		case 'd':
 			debug = 1;
@@ -1438,7 +1411,7 @@ main(int argc, char **argv)
 			break;
 		}
 	}
-	
+
 #if (defined(LIBCMAN_VERSION) && LIBCMAN_VERSION >= 2)
 	ch = cman_admin_init(NULL);
 #else
@@ -1447,7 +1420,7 @@ main(int argc, char **argv)
 	if (!ch) {
 		if (!foreground && !forked) {
 			if (daemon_init(argv[0]) < 0)
-				return -1;
+				goto out;
 			else
 				forked = 1;
 		}
@@ -1468,7 +1441,7 @@ main(int argc, char **argv)
 	while (cman_get_node(ch, CMAN_NODEID_US, &me) < 0) {
 		if (!foreground && !forked) {
 			if (daemon_init(argv[0]) < 0)
-				return -1;
+				goto out;
 			else
 				forked = 1;
 		}
@@ -1482,26 +1455,24 @@ main(int argc, char **argv)
 
         if (debug) {
                 clu_set_loglevel(LOG_DEBUG);
-                ctx.qc_flags |= RF_DEBUG;
         }
-		
-	if (get_config_data(NULL, &ctx, h, 10, &cfh, debug) < 0) {
+
+	if (get_config_data(&ctx, h, 10, &cfh, debug) < 0) {
 		clulog_and_print(LOG_CRIT, "Configuration failed\n");
 		check_stop_cman(&ctx);
-		return -1;
+		goto out;
 	}
-	
+
 	if (ctx.qc_label) {
-		memset(device, 0, sizeof(device));
-		if (find_partitions("/dev",
-				    ctx.qc_label, device,
-				    sizeof(device), 0) != 0) {
+		ret = find_partitions(ctx.qc_label, device, sizeof(device), 0);
+		if (ret < 0) {
 			clulog_and_print(LOG_CRIT, "Unable to match label"
 					 " '%s' to any device\n",
 					 ctx.qc_label);
 			check_stop_cman(&ctx);
-			return -1;
+			goto out;
 		}
+		/* XXX Multiple matches: do we care? */
 
 		if (ctx.qc_device)
 			free(ctx.qc_device);
@@ -1511,18 +1482,29 @@ main(int argc, char **argv)
 		clulog(LOG_INFO, "Quorum Partition: %s Label: %s\n",
 		       ctx.qc_device, ctx.qc_label);
 	} else if (ctx.qc_device) {
-		if (check_device(ctx.qc_device, NULL, NULL) != 0) {
+		if (check_device(ctx.qc_device, NULL, &qh, 0) != 0) {
 			clulog(LOG_CRIT,
 			       "Specified partition %s does not have a "
 			       "qdisk label\n", ctx.qc_device);
 			check_stop_cman(&ctx);
-			return -1;
+			goto out;
+		}
+
+		if (qh.qh_version == VERSION_MAGIC_V2 &&
+                    qh.qh_blksz != rv) {
+			clulog(LOG_CRIT,
+			       "Specified device %s does match kernel's "
+			       "reported sector size (%d != %d)\n",
+			       ctx.qc_device,
+			       ctx.qc_disk.d_blksz, rv);
+			check_stop_cman(&ctx);
+			goto out;
 		}
 	}
 
 	if (!foreground && !forked) {
                 if (daemon_init(argv[0]) < 0)
-			return -1;
+			goto out;
 	}
 	
 	set_priority(ctx.qc_sched, ctx.qc_sched_prio);
@@ -1530,13 +1512,19 @@ main(int argc, char **argv)
 	if (quorum_init(&ctx, ni, MAX_NODES_DISK, h, cfh) < 0) {
 		clulog_and_print(LOG_CRIT, "Initialization failed\n");
 		check_stop_cman(&ctx);
-		return -1;
+		goto out;
 	}
 
+	ret = 0;
+
 	if (!_running)
-		return 0;
+		goto out;
 	
-	cman_register_quorum_device(ctx.qc_ch, ctx.qc_device, ctx.qc_votes);
+	cman_register_quorum_device(ctx.qc_ch,
+				    (ctx.qc_flags&RF_CMAN_LABEL)? 
+				        ctx.qc_cman_label:
+                                        ctx.qc_device,
+				    ctx.qc_votes);
 	/*
 		XXX this always returns -1 / EBUSY even when it works?!!!
 		
@@ -1546,16 +1534,18 @@ main(int argc, char **argv)
 				 "Could not register %s with CMAN; "
 				 "return = %d; error = %s\n",
 				 ctx.qc_device, rv, strerror(errno));
-		return -1;
+		goto out;
 	}
 	*/
-
 	if (quorum_loop(&ctx, ni, MAX_NODES_DISK) == 0)
 		cman_unregister_quorum_device(ctx.qc_ch);
 
 	quorum_logout(&ctx);
+	/* free cman handle to avoid leak in cman */
+out:
+	cman_finish(ctx.qc_ch);
 	qd_destroy(&ctx);
 
-	return 0;
+	return ret;
 }
 
