@@ -1,16 +1,3 @@
-/******************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004-2007 Red Hat, Inc.  All rights reserved.
-**
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-*******************************************************************************
-******************************************************************************/
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -27,7 +14,7 @@
 #include "ccs.h"
 
 #define MAX_METHODS		8
-#define MAX_DEVICES		4
+#define MAX_DEVICES		8
 #define MAX_AGENT_ARGS_LEN	512
 
 #define METHOD_NAME_PATH        "/cluster/clusternodes/clusternode[@name=\"%s\"]/fence/method[%d]/@name"
@@ -219,7 +206,7 @@ static int get_device(int cd, char *victim, char *method, int d, char **device)
 	return error;
 }
 
-static int count_methods(int cd, char *victim)
+int count_methods(int cd, char *victim)
 {
 	char path[256], *name;
 	int error, i;
@@ -279,7 +266,7 @@ static int use_device(int cd, char *victim, char *method, int d,
 	return error;
 }
 
-static void update_cman(char *victim, char *method)
+void update_cman(char *victim, char *method)
 {
 	cman_handle_t ch;
 	struct cman_node node;
@@ -304,10 +291,13 @@ static void update_cman(char *victim, char *method)
 
 int dispatch_fence_agent(char *victim, int force)
 {
+	char good_device[256];
 	char *method = NULL, *device = NULL;
 	char *victim_nodename = NULL;
 	int num_methods, num_devices, m, d, error = -1, cd;
 
+	strcpy(good_device, "UNKNOWN");
+
 	if (force)
 		cd = ccs_force_connect(NULL, 0);
 	else {
@@ -335,16 +325,13 @@ int dispatch_fence_agent(char *victim, int force)
 		if (error == -EBADR) {
 			syslog(LOG_INFO, "ccs connection timed out, "
 				"retrying\n");
-
 			while ((cd = ccs_connect()) < 0)
 				sleep(1);
-			
 			error = get_method(cd, victim, m, &method);
-
+		}
+		
 		if (error)
 			continue;
-		} else if (error)
-			continue;
 
 		/* if num_devices is zero we should return an error */
 		error = -1;
@@ -360,7 +347,7 @@ int dispatch_fence_agent(char *victim, int force)
 			if (error)
 				break;
 
-			update_cman(victim, device);
+			strncpy(good_device, device, sizeof(good_device));
 			free(device);
 			device = NULL;
 		}
@@ -371,8 +358,10 @@ int dispatch_fence_agent(char *victim, int force)
 			free(victim_nodename);
 		free(method);
 
-		if (!error)
+		if (!error) {
+			update_cman(victim, good_device);
 			break;
+		}
 	}
 
 	ccs_disconnect(cd);
