@@ -1,17 +1,3 @@
-/******************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
-**
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-*******************************************************************************
-******************************************************************************/
-
-
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
@@ -24,6 +10,7 @@
 #include <sys/wait.h>
 #include <arpa/inet.h>
 #include <dirent.h>
+#include <limits.h>
 #include <ctype.h>
 #define _GNU_SOURCE
 #include <getopt.h>
