@@ -1,21 +1,3 @@
-/*
-  Copyright Red Hat, Inc. 2004
-
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2, or (at your option) any
-  later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; see the file COPYING.  If not, write to the
-  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
-  MA 02139, USA.
-*/
 #include <pthread.h>
 #include <stdio.h>
 #ifdef NO_CCS
@@ -126,8 +108,8 @@ ccs_lock(void)
 
 
 int
-ccs_unlock(int fd)
 #ifndef NO_CCS
+ccs_unlock(int fd)
 {
 	int ret;
 
@@ -139,6 +121,7 @@ ccs_unlock(int fd)
 	return 0;
 }
 #else
+ccs_unlock(int __attribute__((unused)) fd)
 {
 	xmlFreeDoc(ccs_doc);
 	ccs_doc = NULL;
