@@ -1,20 +1,3 @@
-/*
-  Copyright Red Hat, Inc. 2007
-
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License version 2 as published
-  by the Free Software Foundation.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; see the file COPYING.  If not, write to the
-  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
-  MA 02139, USA.
-*/
 /**
   @file S/Lang event handling & intrinsic functions + vars
  */
@@ -99,7 +82,9 @@ static int
    _user_stop = RG_STOP_USER,		/* From clusvcadm */
    _user_relo = RG_RELOCATE,
    _user_restart = RG_RESTART,
-   _user_migrate = RG_MIGRATE;
+   _user_migrate = RG_MIGRATE,
+   _user_freeze = RG_FREEZE,
+   _user_unfreeze = RG_UNFREEZE;
 
 
 SLang_Intrin_Var_Type rgmanager_vars[] =
@@ -148,6 +133,8 @@ SLang_Intrin_Var_Type rgmanager_vars[] =
 	MAKE_VARIABLE("USER_RELOCATE",	&_user_relo,	SLANG_INT_TYPE, 1),
 	MAKE_VARIABLE("USER_RESTART",	&_user_restart,	SLANG_INT_TYPE, 1),
 	MAKE_VARIABLE("USER_MIGRATE",	&_user_migrate,	SLANG_INT_TYPE, 1),
+	MAKE_VARIABLE("USER_FREEZE",	&_user_freeze,	SLANG_INT_TYPE, 1),
+	MAKE_VARIABLE("USER_UNFREEZE",	&_user_unfreeze,SLANG_INT_TYPE, 1),
 
 	/* Errors */
 	MAKE_VARIABLE("rg_error",	&_rg_err,	SLANG_INT_TYPE, 1),
@@ -261,7 +248,14 @@ sl_service_status(char *svcName)
 		return;
 	}
 
-	state_str = strdup(rg_state_str(svcStatus.rs_state));
+	if (svcStatus.rs_flags & RG_FLAG_FROZEN) {
+		/* Special case: "frozen" is a flag, but user scripts should
+		   treat it as a state. */
+		state_str = strdup(rg_flag_str(RG_FLAG_FROZEN));
+	} else {
+		state_str = strdup(rg_state_str(svcStatus.rs_state));
+	}
+
 	if (!state_str) {
 		SLang_verror(SL_RunTime_Error,
 			     "%s: Failed to duplicate state of %s",
@@ -280,6 +274,21 @@ sl_service_status(char *svcName)
 }
 
 
+/* These can be done by the master node */
+int
+sl_service_freeze(char *svcName)
+{
+	return svc_freeze(svcName);
+}
+
+
+int
+sl_service_unfreeze(char *svcName)
+{
+	return svc_unfreeze(svcName);
+}
+
+
 /**
   (nofailback, restricted, ordered, nodelist) = service_domain_info(svcName);
  */
@@ -573,24 +582,12 @@ push_int_array(int *stuff, int len)
 void
 sl_nodes_online(void)
 {
-	int i, *nodes, nodecount = 0;
+	int x, *nodes = NULL, nodecount = 0;
 
-	cluster_member_list_t *membership = member_list();
-	if (!membership)
-		return;
-	nodes = malloc(sizeof(int) * membership->cml_count);
-	if (!nodes)
+	x = member_online_set(&nodes, &nodecount);
+	if (x < 0 || !nodes || !nodecount)
 		return;
 
-	nodecount = 0;
-	for (i = 0; i < membership->cml_count; i++) {
-		if (membership->cml_members[i].cn_member &&
-		    membership->cml_members[i].cn_nodeid != 0) {
-			nodes[nodecount] = membership->cml_members[i].cn_nodeid;
-			++nodecount;
-		}
-	}
-	free_member_list(membership);
 	push_int_array(nodes, nodecount);
 	free(nodes);
 }
@@ -953,6 +950,10 @@ SLang_Intrin_Fun_Type rgmanager_slang[] =
 	MAKE_INTRINSIC_0("service_start", sl_start_service, SLANG_INT_TYPE),
 	MAKE_INTRINSIC_S("service_status", sl_service_status,
 			 SLANG_VOID_TYPE),
+	MAKE_INTRINSIC_S("service_freeze", sl_service_freeze,
+			 SLANG_INT_TYPE),
+	MAKE_INTRINSIC_S("service_unfreeze", sl_service_unfreeze,
+			 SLANG_INT_TYPE),
 
 	/* Node list manipulation */
 	MAKE_INTRINSIC_0("union", sl_union, SLANG_VOID_TYPE),
